package java.awt.image;

import sun.java2d.cmm.PCMM;
import java.util.Collections;
import java.util.WeakHashMap;
import sun.java2d.cmm.ColorTransform;
import sun.java2d.cmm.CMSManager;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.color.ICC_ColorSpace;
import java.util.Map;
import java.awt.color.ColorSpace;
import java.awt.Transparency;

public abstract class ColorModel implements Transparency
{
    private long pData;
    protected int pixel_bits;
    int[] nBits;
    int transparency;
    boolean supportsAlpha;
    boolean isAlphaPremultiplied;
    int numComponents;
    int numColorComponents;
    ColorSpace colorSpace;
    int colorSpaceType;
    int maxBits;
    boolean is_sRGB;
    protected int transferType;
    private static boolean loaded;
    private static ColorModel RGBdefault;
    static byte[] l8Tos8;
    static byte[] s8Tol8;
    static byte[] l16Tos8;
    static short[] s8Tol16;
    static Map<ICC_ColorSpace, byte[]> g8Tos8Map;
    static Map<ICC_ColorSpace, byte[]> lg16Toog8Map;
    static Map<ICC_ColorSpace, byte[]> g16Tos8Map;
    static Map<ICC_ColorSpace, short[]> lg16Toog16Map;
    
    static void loadLibraries() {
        if (!ColorModel.loaded) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    System.loadLibrary("awt");
                    return null;
                }
            });
            ColorModel.loaded = true;
        }
    }
    
    private static native void initIDs();
    
    public static ColorModel getRGBdefault() {
        if (ColorModel.RGBdefault == null) {
            ColorModel.RGBdefault = new DirectColorModel(32, 16711680, 65280, 255, -16777216);
        }
        return ColorModel.RGBdefault;
    }
    
    public ColorModel(int n) {
        this.transparency = 3;
        this.supportsAlpha = true;
        this.isAlphaPremultiplied = false;
        this.numComponents = -1;
        this.numColorComponents = -1;
        this.colorSpace = ColorSpace.getInstance(1000);
        this.colorSpaceType = 5;
        this.is_sRGB = true;
        this.pixel_bits = n;
        if (n < 1) {
            throw new IllegalArgumentException("Number of bits must be > 0");
        }
        this.numComponents = 4;
        this.numColorComponents = 3;
        this.maxBits = n;
        this.transferType = getDefaultTransferType(n);
        return;
    }
    
    protected ColorModel(int pixel_bits, int[] array, ColorSpace colorSpace, boolean supportsAlpha, boolean isAlphaPremultiplied, int n, int transferType) {
        this.transparency = 3;
        this.supportsAlpha = true;
        this.isAlphaPremultiplied = false;
        this.numComponents = -1;
        this.numColorComponents = -1;
        this.colorSpace = ColorSpace.getInstance(1000);
        this.colorSpaceType = 5;
        this.is_sRGB = true;
        this.colorSpace = colorSpace;
        this.colorSpaceType = colorSpace.getType();
        this.numColorComponents = colorSpace.getNumComponents();
        this.numComponents = this.numColorComponents + (supportsAlpha ? 1 : 0);
        this.supportsAlpha = supportsAlpha;
        if (array.length < this.numComponents) {
            throw new IllegalArgumentException("Number of color/alpha components should be " + this.numComponents + " but length of bits array is " + array.length);
        }
        if (n < 1 || n > 3) {
            throw new IllegalArgumentException("Unknown transparency: " + n);
        }
        if (!this.supportsAlpha) {
            this.isAlphaPremultiplied = false;
            this.transparency = 1;
        }
        else {
            this.isAlphaPremultiplied = isAlphaPremultiplied;
            this.transparency = n;
        }
        this.nBits = (int[])array.clone();
        this.pixel_bits = pixel_bits;
        if (pixel_bits <= 0) {
            throw new IllegalArgumentException("Number of pixel bits must be > 0");
        }
        this.maxBits = 0;
        int i = 0;
        while (i < array.length) {
            if (array[i] < 0) {
                throw new IllegalArgumentException("Number of bits must be >= 0");
            }
            if (this.maxBits < array[i]) {
                this.maxBits = array[i];
            }
            ++i;
        }
        if (this.maxBits == 0) {
            throw new IllegalArgumentException("There must be at least one component with > 0 pixel bits.");
        }
        if (colorSpace != ColorSpace.getInstance(1000)) {
            this.is_sRGB = false;
        }
        this.transferType = transferType;
        return;
    }
    
    public final boolean hasAlpha() {
        return this.supportsAlpha;
    }
    
    public final boolean isAlphaPremultiplied() {
        return this.isAlphaPremultiplied;
    }
    
    public final int getTransferType() {
        return this.transferType;
    }
    
    public int getPixelSize() {
        return this.pixel_bits;
    }
    
    public int getComponentSize(int n) {
        if (this.nBits == null) {
            throw new NullPointerException("Number of bits array is null.");
        }
        return this.nBits[n];
    }
    
    public int[] getComponentSize() {
        if (this.nBits != null) {
            return (int[])this.nBits.clone();
        }
        return null;
    }
    
    @Override
    public int getTransparency() {
        return this.transparency;
    }
    
    public int getNumComponents() {
        return this.numComponents;
    }
    
    public int getNumColorComponents() {
        return this.numColorComponents;
    }
    
    public abstract int getRed(int p0);
    
    public abstract int getGreen(int p0);
    
    public abstract int getBlue(int p0);
    
    public abstract int getAlpha(int p0);
    
    public int getRGB(int n) {
        return this.getAlpha(n) << 24 | this.getRed(n) << 16 | this.getGreen(n) << 8 | this.getBlue(n) << 0;
    }
    
    public int getRed(Object o) {
        int n;
        int n2;
        switch (this.transferType) {
            case 0: {
                byte[] array = (byte[])(byte[])o;
                n = (array[0] & 0xFF);
                n2 = array.length;
                break;
            }
            case 1: {
                short[] array2 = (short[])(short[])o;
                n = (array2[0] & 0xFFFF);
                n2 = array2.length;
                break;
            }
            case 3: {
                int[] array3 = (int[])(int[])o;
                n = array3[0];
                n2 = array3.length;
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        if (n2 == 1) {
            return this.getRed(n);
        }
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public int getGreen(Object o) {
        int n;
        int n2;
        switch (this.transferType) {
            case 0: {
                byte[] array = (byte[])(byte[])o;
                n = (array[0] & 0xFF);
                n2 = array.length;
                break;
            }
            case 1: {
                short[] array2 = (short[])(short[])o;
                n = (array2[0] & 0xFFFF);
                n2 = array2.length;
                break;
            }
            case 3: {
                int[] array3 = (int[])(int[])o;
                n = array3[0];
                n2 = array3.length;
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        if (n2 == 1) {
            return this.getGreen(n);
        }
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public int getBlue(Object o) {
        int n;
        int n2;
        switch (this.transferType) {
            case 0: {
                byte[] array = (byte[])(byte[])o;
                n = (array[0] & 0xFF);
                n2 = array.length;
                break;
            }
            case 1: {
                short[] array2 = (short[])(short[])o;
                n = (array2[0] & 0xFFFF);
                n2 = array2.length;
                break;
            }
            case 3: {
                int[] array3 = (int[])(int[])o;
                n = array3[0];
                n2 = array3.length;
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        if (n2 == 1) {
            return this.getBlue(n);
        }
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public int getAlpha(Object o) {
        int n;
        int n2;
        switch (this.transferType) {
            case 0: {
                byte[] array = (byte[])(byte[])o;
                n = (array[0] & 0xFF);
                n2 = array.length;
                break;
            }
            case 1: {
                short[] array2 = (short[])(short[])o;
                n = (array2[0] & 0xFFFF);
                n2 = array2.length;
                break;
            }
            case 3: {
                int[] array3 = (int[])(int[])o;
                n = array3[0];
                n2 = array3.length;
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        if (n2 == 1) {
            return this.getAlpha(n);
        }
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public int getRGB(Object o) {
        return this.getAlpha(o) << 24 | this.getRed(o) << 16 | this.getGreen(o) << 8 | this.getBlue(o) << 0;
    }
    
    public Object getDataElements(int n, Object o) {
        throw new UnsupportedOperationException("This method is not supported by this color model.");
    }
    
    public int[] getComponents(int n, int[] array, int n2) {
        throw new UnsupportedOperationException("This method is not supported by this color model.");
    }
    
    public int[] getComponents(Object o, int[] array, int n) {
        throw new UnsupportedOperationException("This method is not supported by this color model.");
    }
    
    public int[] getUnnormalizedComponents(float[] array, int n, int[] array2, int n2) {
        if (this.colorSpace == null) {
            throw new UnsupportedOperationException("This method is not supported by this color model.");
        }
        if (this.nBits == null) {
            throw new UnsupportedOperationException("This method is not supported.  Unable to determine #bits per component.");
        }
        if (array.length - n < this.numComponents) {
            throw new IllegalArgumentException("Incorrect number of components.  Expecting " + this.numComponents);
        }
        if (array2 == null) {
            array2 = new int[n2 + this.numComponents];
        }
        if (this.supportsAlpha && this.isAlphaPremultiplied) {
            float n3 = array[n + this.numColorComponents];
            for (int j = 0; j < this.numColorComponents; ++j) {
                array2[n2 + j] = (int)(array[n + j] * (float)((1 << this.nBits[j]) - 1) * n3 + 0.5f);
            }
            array2[n2 + this.numColorComponents] = (int)(n3 * (float)((1 << this.nBits[this.numColorComponents]) - 1) + 0.5f);
            return array2;
        }
        for (int i = 0; i < this.numComponents; ++i) {
            array2[n2 + i] = (int)(array[n + i] * (float)((1 << this.nBits[i]) - 1) + 0.5f);
        }
        return array2;
    }
    
    public float[] getNormalizedComponents(int[] array, int n, float[] array2, int n2) {
        if (this.colorSpace == null) {
            throw new UnsupportedOperationException("This method is not supported by this color model.");
        }
        if (this.nBits == null) {
            throw new UnsupportedOperationException("This method is not supported.  Unable to determine #bits per component.");
        }
        if (array.length - n < this.numComponents) {
            throw new IllegalArgumentException("Incorrect number of components.  Expecting " + this.numComponents);
        }
        if (array2 == null) {
            array2 = new float[this.numComponents + n2];
        }
        if (this.supportsAlpha && this.isAlphaPremultiplied) {
            float n3 = (float)array[n + this.numColorComponents] / (float)((1 << this.nBits[this.numColorComponents]) - 1);
            if (n3 != 0.0f) {
                for (int k = 0; k < this.numColorComponents; ++k) {
                    array2[n2 + k] = (float)array[n + k] / (n3 * (float)((1 << this.nBits[k]) - 1));
                }
            }
            else {
                for (int j = 0; j < this.numColorComponents; ++j) {
                    array2[n2 + j] = 0.0f;
                }
            }
            array2[n2 + this.numColorComponents] = n3;
            return array2;
        }
        else {
            for (int i = 0; i < this.numComponents; ++i) {
                array2[n2 + i] = (float)array[n + i] / (float)((1 << this.nBits[i]) - 1);
            }
            return array2;
        }
    }
    
    public int getDataElement(int[] array, int n) {
        throw new UnsupportedOperationException("This method is not supported by this color model.");
    }
    
    public Object getDataElements(int[] array, int n, Object o) {
        throw new UnsupportedOperationException("This method has not been implemented for this color model.");
    }
    
    public int getDataElement(float[] array, int n) {
        return this.getDataElement(this.getUnnormalizedComponents(array, n, null, 0), 0);
    }
    
    public Object getDataElements(float[] array, int n, Object o) {
        return this.getDataElements(this.getUnnormalizedComponents(array, n, null, 0), 0, o);
    }
    
    public float[] getNormalizedComponents(Object o, float[] array, int n) {
        return this.getNormalizedComponents(this.getComponents(o, null, 0), 0, array, n);
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof ColorModel)) {
            return false;
        }
        ColorModel colorModel = (ColorModel)o;
        if (this == colorModel) {
            return true;
        }
        if (this.supportsAlpha != colorModel.hasAlpha() || this.isAlphaPremultiplied != colorModel.isAlphaPremultiplied() || this.pixel_bits != colorModel.getPixelSize() || this.transparency != colorModel.getTransparency() || this.numComponents != colorModel.getNumComponents()) {
            return false;
        }
        int[] componentSize = colorModel.getComponentSize();
        if (this.nBits != null && componentSize != null) {
            int i = 0;
            while (i < this.numComponents) {
                if (this.nBits[i] != componentSize[i]) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        return this.nBits == null && componentSize == null;
    }
    
    @Override
    public int hashCode() {
        int n = (this.supportsAlpha ? 2 : 3) + (this.isAlphaPremultiplied ? 4 : 5) + this.pixel_bits * 6 + this.transparency * 7 + this.numComponents * 8;
        if (this.nBits != null) {
            for (int i = 0; i < this.numComponents; ++i) {
                n += this.nBits[i] * (i + 9);
            }
        }
        return n;
    }
    
    public final ColorSpace getColorSpace() {
        return this.colorSpace;
    }
    
    public ColorModel coerceData(WritableRaster writableRaster, boolean b) {
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public boolean isCompatibleRaster(Raster raster) {
        throw new UnsupportedOperationException("This method has not been implemented for this ColorModel.");
    }
    
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public SampleModel createCompatibleSampleModel(int n, int n2) {
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public boolean isCompatibleSampleModel(SampleModel sampleModel) {
        throw new UnsupportedOperationException("This method is not supported by this color model");
    }
    
    public void finalize() {
    }
    
    public WritableRaster getAlphaRaster(WritableRaster writableRaster) {
        return null;
    }
    
    @Override
    public String toString() {
        return new String("ColorModel: #pixelBits = " + this.pixel_bits + " numComponents = " + this.numComponents + " color space = " + this.colorSpace + " transparency = " + this.transparency + " has alpha = " + this.supportsAlpha + " isAlphaPre = " + this.isAlphaPremultiplied);
    }
    
    static int getDefaultTransferType(int n) {
        if (n <= 8) {
            return 0;
        }
        if (n <= 16) {
            return 1;
        }
        if (n <= 32) {
            return 3;
        }
        return 32;
    }
    
    static boolean isLinearRGBspace(ColorSpace colorSpace) {
        return colorSpace == CMSManager.LINEAR_RGBspace;
    }
    
    static boolean isLinearGRAYspace(ColorSpace colorSpace) {
        return colorSpace == CMSManager.GRAYspace;
    }
    
    static byte[] getLinearRGB8TosRGB8LUT() {
        if (ColorModel.l8Tos8 == null) {
            ColorModel.l8Tos8 = new byte[256];
            float n2;
            for (int i = 0; i <= 255; ColorModel.l8Tos8[i] = (byte)Math.round(n2 * 255.0f), ++i) {
                float n = (float)i / 255.0f;
                if (n <= 0.0031308f) {
                    n2 = n * 12.92f;
                }
                else {
                    n2 = 1.055f * (float)Math.pow((double)n, 0.4166666666666667) - 0.055f;
                }
            }
        }
        return ColorModel.l8Tos8;
    }
    
    static byte[] getsRGB8ToLinearRGB8LUT() {
        if (ColorModel.s8Tol8 == null) {
            ColorModel.s8Tol8 = new byte[256];
            float n2;
            for (int i = 0; i <= 255; ColorModel.s8Tol8[i] = (byte)Math.round(n2 * 255.0f), ++i) {
                float n = (float)i / 255.0f;
                if (n <= 0.04045f) {
                    n2 = n / 12.92f;
                }
                else {
                    n2 = (float)Math.pow((double)((n + 0.055f) / 1.055f), 2.4);
                }
            }
        }
        return ColorModel.s8Tol8;
    }
    
    static byte[] getLinearRGB16TosRGB8LUT() {
        if (ColorModel.l16Tos8 == null) {
            ColorModel.l16Tos8 = new byte[65536];
            float n2;
            for (int i = 0; i <= 65535; ColorModel.l16Tos8[i] = (byte)Math.round(n2 * 255.0f), ++i) {
                float n = (float)i / 65535.0f;
                if (n <= 0.0031308f) {
                    n2 = n * 12.92f;
                }
                else {
                    n2 = 1.055f * (float)Math.pow((double)n, 0.4166666666666667) - 0.055f;
                }
            }
        }
        return ColorModel.l16Tos8;
    }
    
    static short[] getsRGB8ToLinearRGB16LUT() {
        if (ColorModel.s8Tol16 == null) {
            ColorModel.s8Tol16 = new short[256];
            float n2;
            for (int i = 0; i <= 255; ColorModel.s8Tol16[i] = (short)Math.round(n2 * 65535.0f), ++i) {
                float n = (float)i / 255.0f;
                if (n <= 0.04045f) {
                    n2 = n / 12.92f;
                }
                else {
                    n2 = (float)Math.pow((double)((n + 0.055f) / 1.055f), 2.4);
                }
            }
        }
        return ColorModel.s8Tol16;
    }
    
    static byte[] getGray8TosRGB8LUT(ICC_ColorSpace icc_ColorSpace) {
        if (isLinearGRAYspace(icc_ColorSpace)) {
            return getLinearRGB8TosRGB8LUT();
        }
        if (ColorModel.g8Tos8Map != null) {
            byte[] array = (byte[])ColorModel.g8Tos8Map.get(icc_ColorSpace);
            if (array != null) {
                return array;
            }
        }
        byte[] array2 = new byte[256];
        for (int i = 0; i <= 255; ++i) {
            array2[i] = (byte)i;
        }
        ColorTransform[] array3 = new ColorTransform[2];
        PCMM module = CMSManager.getModule();
        ICC_ColorSpace icc_ColorSpace2 = (ICC_ColorSpace)ColorSpace.getInstance(1000);
        array3[0] = module.createTransform(icc_ColorSpace.getProfile(), -1, 1);
        array3[1] = module.createTransform(icc_ColorSpace2.getProfile(), -1, 2);
        byte[] colorConvert = module.createTransform(array3).colorConvert(array2, (byte[])null);
        for (int j = 0, n = 2; j <= 255; ++j, n += 3) {
            array2[j] = colorConvert[n];
        }
        if (ColorModel.g8Tos8Map == null) {
            ColorModel.g8Tos8Map = Collections.synchronizedMap(new WeakHashMap(2));
        }
        ColorModel.g8Tos8Map.put(icc_ColorSpace, array2);
        return array2;
    }
    
    static byte[] getLinearGray16ToOtherGray8LUT(ICC_ColorSpace icc_ColorSpace) {
        if (ColorModel.lg16Toog8Map != null) {
            byte[] array = (byte[])ColorModel.lg16Toog8Map.get(icc_ColorSpace);
            if (array != null) {
                return array;
            }
        }
        short[] array2 = new short[65536];
        for (int i = 0; i <= 65535; ++i) {
            array2[i] = (short)i;
        }
        ColorTransform[] array3 = new ColorTransform[2];
        PCMM module = CMSManager.getModule();
        array3[0] = module.createTransform(((ICC_ColorSpace)ColorSpace.getInstance(1003)).getProfile(), -1, 1);
        array3[1] = module.createTransform(icc_ColorSpace.getProfile(), -1, 2);
        short[] colorConvert = module.createTransform(array3).colorConvert(array2, (short[])null);
        byte[] array4 = new byte[65536];
        for (int j = 0; j <= 65535; ++j) {
            array4[j] = (byte)(int)((float)(colorConvert[j] & 0xFFFF) * 0.0038910506f + 0.5f);
        }
        if (ColorModel.lg16Toog8Map == null) {
            ColorModel.lg16Toog8Map = Collections.synchronizedMap(new WeakHashMap(2));
        }
        ColorModel.lg16Toog8Map.put(icc_ColorSpace, array4);
        return array4;
    }
    
    static byte[] getGray16TosRGB8LUT(ICC_ColorSpace icc_ColorSpace) {
        if (isLinearGRAYspace(icc_ColorSpace)) {
            return getLinearRGB16TosRGB8LUT();
        }
        if (ColorModel.g16Tos8Map != null) {
            byte[] array = (byte[])ColorModel.g16Tos8Map.get(icc_ColorSpace);
            if (array != null) {
                return array;
            }
        }
        short[] array2 = new short[65536];
        for (int i = 0; i <= 65535; ++i) {
            array2[i] = (short)i;
        }
        ColorTransform[] array3 = new ColorTransform[2];
        PCMM module = CMSManager.getModule();
        ICC_ColorSpace icc_ColorSpace2 = (ICC_ColorSpace)ColorSpace.getInstance(1000);
        array3[0] = module.createTransform(icc_ColorSpace.getProfile(), -1, 1);
        array3[1] = module.createTransform(icc_ColorSpace2.getProfile(), -1, 2);
        short[] colorConvert = module.createTransform(array3).colorConvert(array2, (short[])null);
        byte[] array4 = new byte[65536];
        for (int j = 0, n = 2; j <= 65535; ++j, n += 3) {
            array4[j] = (byte)(int)((float)(colorConvert[n] & 0xFFFF) * 0.0038910506f + 0.5f);
        }
        if (ColorModel.g16Tos8Map == null) {
            ColorModel.g16Tos8Map = Collections.synchronizedMap(new WeakHashMap(2));
        }
        ColorModel.g16Tos8Map.put(icc_ColorSpace, array4);
        return array4;
    }
    
    static short[] getLinearGray16ToOtherGray16LUT(ICC_ColorSpace icc_ColorSpace) {
        if (ColorModel.lg16Toog16Map != null) {
            short[] array = (short[])ColorModel.lg16Toog16Map.get(icc_ColorSpace);
            if (array != null) {
                return array;
            }
        }
        short[] array2 = new short[65536];
        for (int i = 0; i <= 65535; ++i) {
            array2[i] = (short)i;
        }
        ColorTransform[] array3 = new ColorTransform[2];
        PCMM module = CMSManager.getModule();
        array3[0] = module.createTransform(((ICC_ColorSpace)ColorSpace.getInstance(1003)).getProfile(), -1, 1);
        array3[1] = module.createTransform(icc_ColorSpace.getProfile(), -1, 2);
        short[] colorConvert = module.createTransform(array3).colorConvert(array2, (short[])null);
        if (ColorModel.lg16Toog16Map == null) {
            ColorModel.lg16Toog16Map = Collections.synchronizedMap(new WeakHashMap(2));
        }
        ColorModel.lg16Toog16Map.put(icc_ColorSpace, colorConvert);
        return colorConvert;
    }
    
    static {
        ColorModel.loaded = false;
        loadLibraries();
        initIDs();
        ColorModel.l8Tos8 = null;
        ColorModel.s8Tol8 = null;
        ColorModel.l16Tos8 = null;
        ColorModel.s8Tol16 = null;
        ColorModel.g8Tos8Map = null;
        ColorModel.lg16Toog8Map = null;
        ColorModel.g16Tos8Map = null;
        ColorModel.lg16Toog16Map = null;
    }
}
